/*
    Sweeper ... 
	This file is a part of sweeper, a program for identifying
	and estimating regression models, selecting from many possible 
	predictors without overfitting.

    Copyright (C) 1999,2002  Robert A. Stine

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License in the file COPYING for more details.
*/
/*  $Id: bennett.c,v 1.2 2002/05/06 02:45:08 bob Exp $

    26 Feb 02 ... Created

*/

#include "bennett.h"

#include <math.h>
#include <assert.h>

namespace {
  inline double abs_val(double x)
  { return (x>=0) ? x  : -x; }
}

double
bennett_lower_bound (double beta, double a, double b, double pValue)
{
  // assert(b > 0);
  if (b > 0.0)
    {
      double m = a/b;
      double beta0 = abs_val(beta);
      double p = bennett_p_value(beta0/b,m);
      if (p > pValue)
	return 0.0;
      else {
	while (bennett_p_value(beta0/b,m) < pValue)
	  beta0 -= b;
	beta0 += .1 * b;
	while (bennett_p_value(beta0/b,m) > pValue)
	  beta0 += .1 * b;
	return beta-beta0;
      }
    }
  else
    return 0.0;
}

double
bennett_p_value (double bHat, double a, double b)
{
  if (b > 0.0)
    return bennett_p_value (bHat/b, a/b);
  else
    return 1.0;
}


double
bennett_p_value (double t, double m)  
{
  return exp( t/m -
	      (t/m + 1.0/(m*m)) * log(1.0 + m * t));
}
