/*
    Sweeper ... 
	This file is a part of sweeper, a program for identifying
	and estimating regression models, selecting from many possible 
	predictors without overfitting.

    Copyright (C) 1999,2002  Robert A. Stine

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License in the file COPYING for more details.
*/
/*
   13 Aug 01 ... Add file io functions.
   30 Oct 98 ... Initialize function added, more functional style added
   26 Oct 98 ... Pure pointer, functional style
   15 Oct 98 ... Created.
*/

#ifndef _MATRIX_
#define _MATRIX_  

////////////////////////////////////////////////////////////////////////////////

typedef struct aMatrix
{
  double **pCols;
  long nRows, nCols;
} aMatrix;

typedef struct aMatrix *Matrix;

////////////////////////////////////////////////////////////////////////////////

Matrix 	NewMatrix (long nRows, long nCols, char *creator);

Matrix 	IdentityMatrix (long dim, char *creator);
Matrix 	CopyMatrix (Matrix m, char *creator);

int     InitializeMatrix (Matrix m, double initialValue);

void  	DeleteMatrix (Matrix m);


//  file io

Matrix
RestoreMatrixFromFile (FILE *file, int nPad, char *creator);

long
SaveMatrixToFile (Matrix m, FILE *file);


//  access to matrix elements, columns

void    MatrixDimensions (Matrix m, long *nRows, long *nCols);

double 	MatrixElement (Matrix m, long row, long col);
double *MatrixColumn (Matrix m, long col);	// pointer into actual array, not copy

int 	AssignMatrixElement (Matrix m, long row, long col, double x);

int     InsertMatrix (Matrix src, Matrix dest);  // puts src at origin of dest

//  Element by element operations

int     IncrementMatrixElement	(Matrix m, long row, long col, double x);
int 	DecrementMatrixElement  (Matrix m, long row, long col, double x);

int 	MultiplyMatrixElement   (Matrix m, long row, long col, double factor);
int 	DivideMatrixElement	(Matrix m, long row, long col, double divisor);

int     MultiplyMatrixByConstant (Matrix m, double x);


//  Matrix operations; multiplication, division

double	TraceOfMatrix (Matrix m);

Matrix 	MatrixProduct (Matrix a, Matrix b, char *creator);

Matrix 	LUFactor (Matrix mat, long *permutation, double *parity, char *creator);
void 	LUSolve (Matrix factor, long *permutation, double b[]);

Matrix	InverseMatrix (Matrix m, char *creator);

double  QuadraticForm (Matrix m, double *x);          // x'Mx

Matrix  TriangularQuadraticForm (Matrix u, Matrix m); // u m u', u up triagular


//  Stepwise calculations

int ForwardStepwise (Matrix XpX, long nSteps, long *cols);
/*
	Which of the columns to add will have the most impact.
	Assume that Y is the first col of the X'X input array.
*/

int ReverseStepwise (Matrix XpX, long nSteps, long *cols);
/*
	Assuming all have been swept, restoring which of the 
	previously swept columns will have the least impact if 
	removed from the fitted model.
*/

int ForwardSweepMatrix(Matrix m, long col);
int ReverseSweepMatrix(Matrix m, long col);

void PrintMatrix(Matrix m, char *msg);

#endif

