/*
    Sweeper ... 
	This file is a part of sweeper, a program for identifying
	and estimating regression models, selecting from many possible 
	predictors without overfitting.

    Copyright (C) 1999,2002  Robert A. Stine

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License in the file COPYING for more details.
*/

// $Id: sorter.c,v 1.7 2002/05/06 02:45:08 bob Exp $

#include "sorter.h"

// first is RSS, second is conservative p-value

/* This version sorts on the first key element alone... better be discounted one 
bool
Key_sorter::operator()(Key lhs, Key rhs)  // act like lhs > rhs
{
  // always sort on the first which is the guaranteed dRSS
  return (lhs.first > rhs.first);
}
*/

/* This was version sorts on only p-values
bool
Key_sorter::operator()(Key lhs, Key rhs) 
{
  bool lhs_sig = (lhs.second < s_threshold); // smaller since threshold is a p-val
  bool rhs_sig = (rhs.second < s_threshold); //  "
  if(lhs_sig == rhs_sig)
  {
      return lhs.second < rhs.second;
  }
  else
  {
    return lhs_sig;
  }
}
*/

/* This version sorts on dRSS among those that are significant */

bool
Key_sorter::operator()(Key lhs, Key rhs)  // act like: (lhs > rhs)
{
  bool lhs_sig = (lhs.second < s_threshold); // smaller since its a p-val
  bool rhs_sig = (rhs.second < s_threshold); //  "
  if(lhs_sig == rhs_sig) {
    if (lhs_sig) // sort on the rss if both are significant
      return lhs.first > rhs.first;
    else // sort on the p-value
      return lhs.second < rhs.second;
  }
  else {
    return lhs_sig;
  }
}


double Key_sorter::s_threshold = 0.0;


std::ostream&
operator<<(std::ostream& ostrm, std::pair<Key,int> obj)
{
  ostrm << "[ rss = " << obj.first.first;
  if(obj.first.second > Key_sorter::s_threshold)
    ostrm << ", F = "   << obj.first.second;
  else
    ostrm << ", f = "   << obj.first.second;
  ostrm << " ] = "    << obj.second;
  return ostrm;
}
