/*
    Sweeper ... 
	This file is a part of sweeper, a program for identifying
	and estimating regression models, selecting from many possible 
	predictors without overfitting.

    Copyright (C) 1999,2002  Robert A. Stine

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License in the file COPYING for more details.
*/
// $Id: sweeper.test.c,v 1.11 2000/10/30 22:30:55 bob Exp $

#include <stdio.h>
#include <string.h>

#include "parser.h"
#include "fileOps.h"
#include "random.h"

void Wait (void);

void Wait (void)
{
	char ch;
	ch = (char) getchar();
}


// --------------------------------------------------------------------------

#define BUFFER 100
#define SEED0  61467
#define GENTYPE 0

int GenerateTestDataFiles (char *path);
	

int GenerateTestDataFiles (char *path)
{
  char  txtName[BUFFER];
  int
    cols,
    nGenLines = 1000,
    nGenCols  = 1+40,        // 1 for response, plus predictors
    predCol   = 2;           // ensure in range of predictors
  float predEffect = 0.45;    // use 0.45 for binary data, 5 for regr form
  clock_t startTime, stopTime;

  InitializeRandomGenerator(GENTYPE, SEED0);
  
  printf ("\n -------------  Generate test data file  ----------- \n");
  startTime = clock();
  strcpy (txtName, path);  strcat(txtName, "test/est.dat");
  // --- test data with pure noise, continuous response --- 
  // seed = WriteRandomFile(txtName, nGenLines, nGenCols, SEED0);
  // --- test data with regr form ---
  // cols = WriteRegrFileWithSignal(txtName, nGenLines, nGenCols, predCol, predEffect);
  // --- test data has strong signal in predictor predCol ---
  cols = WriteBooleanFileWithSignal(txtName, nGenLines, nGenCols, predCol, predEffect);
  stopTime = clock();
  printf ("Wrote %d lines with %d columns to %s\n", nGenLines, cols, txtName);
  PrintDuration (startTime, stopTime);
  
/*    printf ("\n -------------  Generate test validation data file  ----------- \n"); */
/*    startTime = clock(); */
/*    strcpy (txtName, path);  strcat(txtName, "test/val.dat"); */
/*    cols = WriteRegrFileWithSignal(txtName, nGenLines, nGenCols, predCol, predEffect); */
/*    // cols = WriteBooleanFileWithSignal(txtName, nGenLines, nGenCols, predCol, predEffect); */
/*    stopTime = clock(); */
/*    printf ("Wrote %d lines with %d columns to %s\n", nGenLines, cols, txtName); */
/*    PrintDuration (startTime, stopTime); */
  
  return 1;
}


// --------------------------------------------------------------------------

int main(void)
{
  char *basePath = "/home/bob/test_dir/sweeper/";
  int result;
  int useInteraction = 1;
  int useWts = 1;
     	
  result = GenerateTestDataFiles (basePath);
  if (0 == result)
  { printf ("ERROR in generating test data; returning.\n");
    return -1;
  } else
  {
    printf ("\n Use cmds  `b'  then  `i'  to build BDF and init SM in /test.\n");
  }
  
  // start the reader
  Parser (basePath, useInteraction, useWts);
  
  // close up and return
  printf ("\n\nFINISHED ...\n\n");
  return 1;
}
