/*
    Sweeper ... 
	This file is a part of sweeper, a program for identifying
	and estimating regression models, selecting from many possible 
	predictors without overfitting.

    Copyright (C) 1999,2002  Robert A. Stine

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License in the file COPYING for more details.
*/
/*
	24 Oct 98 ... Created.
*/

#include <stdio.h>
#include <stdlib.h>

#include "taggedList.h"


// Internal

static TaggedListNode *NewTaggedListNode (Tag tag, void *object);

static TaggedListNode *NewTaggedListNode (Tag tag, void *object)
{
	TaggedListNode *pListNode;
	
	// fprintf(stderr, "\n    Allocating new tagged list node.\n");
	pListNode = new TaggedListNode;
	if (NULL != pListNode)
	{	pListNode->object = object;
		pListNode->tag = tag;
		pListNode->next = NULL;
	}
	return(pListNode);
}



//  NewTaggedList

TaggedList * NewTaggedList(void)
{	TaggedListNode *pStart, *pStop;
	
	// objects in head and tail nodes are null
	pStart = NewTaggedListNode(-1, NULL);
	pStop  = NewTaggedListNode(-1, NULL);
	if (NULL == pStart || NULL == pStop)
		return NULL;
	else
	{	pStart->next = pStop;
		return pStart;
	}
}


//  InsertIntoTaggedList   (at head of list)

TaggedListNode *InsertIntoTaggedList(TaggedList *list, Tag tag, void *object)
{	TaggedListNode * pNewNode;

	pNewNode = NewTaggedListNode(tag, object);
	if (pNewNode != NULL)
	{	pNewNode->next = list->next;
		list->next = pNewNode;
	}
	return pNewNode;
}


// Check to see if has a particular tag

TaggedListNode	*TaggedListMember(TaggedList *list, Tag tag)
{	TaggedListNode *listNode = list;

	while (NULL != listNode->next && listNode->next->tag != tag)
	{	listNode = listNode->next; }
	return (listNode->next);
}


//  ExtractFromTaggedList 		(also deletes the node)

void *ExtractFromTaggedList(TaggedList *list, Tag tag)
{	TaggedListNode *listNode = list, *marker;
	void *object = NULL;

	while (NULL != listNode->next && listNode->next->tag != tag)
	{	listNode = listNode->next; }
	if (NULL != listNode->next)
	{	marker = listNode->next;
		object = marker->object;
		listNode->next = marker->next;
		delete(marker);
	}
	// PrintTags(list);
	return (object);
}

//  ObjectInTaggedList

void *ObjectInTaggedList(TaggedList *list, Tag tag)
{	TaggedListNode *listNode = list;

	// move to first content node in list
	listNode = listNode->next;
	// does the list have this node?
	while (listNode->next && listNode->tag != tag)
		listNode = listNode->next;
	// object in last node defaults to null
	return listNode->object;
}


//  PrintTaggedList 

void PrintTags (TaggedList *list)
{	TaggedListNode *pNode;
	
	printf("\nTagged List: ");
	pNode = list->next;
	if (pNode->next == NULL)
	 	printf (" is empty.");
	else
		while (pNode->next)
		{	printf (" [%4ld] ", pNode->tag);
			pNode = pNode->next;
		}
	printf("\n");
}

				
void PrintTaggedList(TaggedList *list, void (*printObj)(void *))
{	TaggedListNode *pNode;
	
	printf("\nTagged List");
	pNode = list->next;
	if (pNode->next == NULL)
	 	printf (" is empty.\n");
	else
	{	printf ("\n");
		while (pNode->next)
		{	printf ("Tag [%4ld] ", pNode->tag);
			printObj(pNode->object);
			pNode = pNode->next;
		}
	}
}

