/*
    Sweeper ... 
	This file is a part of sweeper, a program for identifying
	and estimating regression models, selecting from many possible 
	predictors without overfitting.

    Copyright (C) 1999,2002  Robert A. Stine

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License in the file COPYING for more details.
*/
// $Id: vector.h,v 1.14 2002/03/15 15:16:13 bob Exp $

/*
   30 Jun 00 ... Square a vector, element multiply
    4 Oct 99 ... Add functions to save and restore from a file.
   30 Oct 98 ... Initialize function added, more functional style added
   26 Oct 98 ... Pure pointer, functional style, with NewVector as creator.
   15 Oct 98 ... Created.
*/
#ifndef _VECTOR_
#define _VECTOR_

// Turn off range checks   #define CHECK_BOUNDS

//////////////////////////////////////////////////////////////////////

typedef struct av
{	double *ptr;
	long	len;
} aVector;

typedef aVector *Vector;

/*  IndexedVector ---
	This type binds the pair
		(variable index, vector (as of covariances))
	binding together an index along with the associated vector.

	Note that an IndexedVector *is* a record, not the pointer
	to such a record.
*/

typedef struct indexedVector
{	long columnIndex;
	Vector theVector;
} IndexedVector;

/////////////////////////////////////////////////////////////////////

Vector	NewVector (long n, char *creator);
Vector	NewUniformVector (long n, char *creator);
Vector  NewNormalVector (long n, char *creator);

/* Initialize random generator first! */

Vector 	RestoreVectorFromFile (FILE *file, char *creator);
int     SaveVectorToFile (Vector v, FILE *file);

void	DeleteVector(Vector v);

Vector	CopyVector (Vector vec, char *creator);
int     CopyVectorToDest (Vector src, Vector dest);

int     CopyIndexedVector (IndexedVector *src, IndexedVector *dest);


int	InitializeVector (Vector v, double initialValue);
long	VectorLength (Vector v);

#ifdef CHECK_BOUNDS
double	VectorElement (Vector v, long index);
int	AssignVectorElement (Vector v, long index, double x);
int	IncrementVectorElement	(Vector v, long index, double x);
int	DecrementVectorElement  (Vector v, long index, double x);
#else
inline double
VectorElement (Vector v, long index)
{ return v->ptr[index]; }

inline int
AssignVectorElement (Vector v, long index, double x)
{ v->ptr[index] = x; return 1; }

inline int
IncrementVectorElement	(Vector v, long i, double x)
{ v->ptr[i] = v->ptr[i] + x; return 1; }

inline int
DecrementVectorElement  (Vector v, long i, double x)
{ v->ptr[i] = v->ptr[i] - x; return 1; }
#endif

int	DivideVectorElement	(Vector v, long index, double divisor);
int	MultiplyVectorElement	(Vector v, long index, double factor);
int     MultiplyVectors (Vector a, Vector b); // result in b

double	VectorMean (Vector v);
double 	VectorSD (Vector v);
void	CenterVector (Vector v, double center);
void	ScaleVector (Vector v, double scale);
void    SquareVector (Vector v);
void	StandardizeVector (Vector v);

double	VectorDotProduct (Vector a, Vector b);
double	VectorCorrelation (Vector a, Vector b);

double	VectorDistance(Vector a, Vector b, int norm);


void	PrintVector (Vector v, char *msg);     // one per row

void	PrintVec (void *v);	// across row, no message...
                                        // void for use as print function in lists
void	CompactPrintVector(Vector v, char *msg);	// across row

void	PrintRowVector (Vector v, long index);

void    PrintRawVector (long length, double *ptr, char *msg);

#endif











